#define BC_FREE_SLIP 1
#define BC_NO_SLIP 2
#define BC_OUTFLOW 3
#define BC_PERIODIC 4

/* Set boundary values of u, v and p according to the boundary conditions.
 * The BC flags are as follows:
 *   1: Free-slip condition
 *   2: No-slip condition
 *   3: Outflow condition
 *   4: Periodic condition
 */
void set_boundary_conditions(int w_left, int w_right, int w_top, int w_bottom,
        double **P, double **U, double **V, int nx, int ny)
{
    int i, j;

    /* vertical boundaries (left and right) */
    for (j=1; j<ny+1; j++) {

        if (w_left == BC_FREE_SLIP) {
            U[0][j] = 0.0;
            V[0][j] = V[1][j];
        } else if (w_left == BC_NO_SLIP) {
            U[0][j] = 0.0;
            V[0][j] = -1.0*V[1][j];
        } else if (w_left == BC_OUTFLOW) {
            U[0][j] = U[1][j];
            V[0][j] = V[1][j];
        } else if (w_left == BC_PERIODIC) {
            U[0][j] = U[nx-1][j];
            V[0][j] = V[nx-1][j];
            V[1][j] = V[nx][j];
            P[1][j] = P[nx][j];
        }

        if (w_right == BC_FREE_SLIP) {
            U[nx][j] = 0.0;
            V[nx+1][j] = V[nx][j];
        } else if (w_right == BC_NO_SLIP) {
            U[nx][j] = 0.0;
            V[nx+1][j] = -1.0*V[nx][j];
        } else if (w_right == BC_OUTFLOW) {
            U[nx][j] = U[nx-1][j];
            V[nx+1][j] = V[nx][j];
        } else if (w_right == BC_PERIODIC) {
            U[nx][j] = U[1][j];
            V[nx+1][j] = V[2][j];
        }
    }

    /* horizontal boundaries (bottom and top) */
    for (i=1; i<nx+1; i++) {

        if (w_bottom == BC_FREE_SLIP) {
            U[i][0] = U[i][1];
            V[i][0] = 0.0;
        } else if (w_bottom == BC_NO_SLIP) {
            U[i][0] = -1.0*U[i][1];
            V[i][0] = 0.0;
        } else if (w_bottom == BC_OUTFLOW) {
            U[i][0] = U[i][1];
            V[i][0] = V[i][1];
        } else if (w_bottom == BC_PERIODIC) {
            U[i][0] = U[i][ny-1];
            U[i][1] = U[i][nx-1];
            V[i][0] = V[i][ny-1];
            P[i][1] = P[i][ny];
        }

        if (w_top == BC_FREE_SLIP) {
            U[i][ny+1] = U[i][ny];
            V[i][ny] = 0.0;
        } else if (w_top == BC_NO_SLIP) {
            U[i][ny+1] = -1.0*U[i][ny];
            V[i][ny] = 0.0;
        } else if (w_top == BC_OUTFLOW) {
            U[i][ny+1] = U[i][ny];
            V[i][ny] = V[i][ny-1];
        } else if (w_top == BC_PERIODIC) {
            U[i][ny+1] = U[i][2];
            V[i][ny] = V[i][1];
        }
    }
}
