#include <stdio.h>
#include <stdlib.h>
#include "utility.h"

/* Read the variable values from a file on disk */
int read_file(char *path,
        double *t, double *t_end, double *t_file, double *tau, int *itermax,
        double *epsilon, double *omega, double *gamma,
        double *gx, double *gy, double *re,
        int *w_left, int *w_right, int *w_top, int *w_bottom,
        double *dx, double *dy,
        int *nx, int *ny,
        double ***P, double ***U, double ***V)
{
    int i, j;
    double tmp;
    FILE *fp;
    if ((fp = fopen(path, "rb")) == NULL) {
        fprintf(stderr, "read_file: Could not open file %s\n", path);
        return 1;
    } else {

        if (1 != fread(t, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(t_end, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(t_file, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(tau, sizeof(double), 1, fp))
            exit(1);

        if (1 != fread(itermax, sizeof(int), 1, fp))
            exit(1);
        if (1 != fread(epsilon, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(omega, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(gamma, sizeof(double), 1, fp))
            exit(1);

        if (1 != fread(gx, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(gy, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(re, sizeof(double), 1, fp))
            exit(1);

        if (1 != fread(w_left, sizeof(int), 1, fp))
            exit(1);
        if (1 != fread(w_right, sizeof(int), 1, fp))
            exit(1);
        if (1 != fread(w_top, sizeof(int), 1, fp))
            exit(1);
        if (1 != fread(w_bottom, sizeof(int), 1, fp))
            exit(1);

        if (1 != fread(dx, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(dy, sizeof(double), 1, fp))
            exit(1);
        if (1 != fread(nx, sizeof(int), 1, fp))
            exit(1);
        if (1 != fread(ny, sizeof(int), 1, fp))
            exit(1);

        allocate_memory(P, U, V, *nx, *ny);
        
        for (i=0; i<*nx+2; i++) {
            for (j=0; j<*ny+2; j++) {
                if (fread(&tmp, sizeof(double), 1, fp) != 1)
                    exit(1);
                (*P)[i][j] = tmp;
            }
        }

        for (i=0; i<*nx+2; i++) {
            for (j=0; j<*ny+2; j++) {
                if (fread(&tmp, sizeof(double), 1, fp) != 1)
                    exit(1);
                (*U)[i][j] = tmp;
            }
        }

        for (i=0; i<*nx+2; i++) {
            for (j=0; j<*ny+2; j++) {
                if (fread(&tmp, sizeof(double), 1, fp) != 1)
                    exit(1);
                (*V)[i][j] = tmp;
            }
        }

        fclose(fp);
    }
    return 0;
}

/* Write the variable values to a file on disk */
int write_file(char *path,
        double *t, double *t_end, double *t_file, double *tau, int *itermax,
        double *epsilon, double *omega, double *gamma,
        double *gx, double *gy, double *re,
        int *w_left, int *w_right, int *w_top, int *w_bottom,
        double *dx, double *dy,
        int *nx, int *ny,
        double ***P, double ***U, double ***V)
{
    int i, j;
    double tmp;
    FILE *fp;
    if ((fp = fopen(path, "wb")) == NULL) {
        fprintf(stderr, "write_file: Could not open file %s\n", path);
        return 1;
    } else {

        fwrite(t, sizeof(double), 1, fp);
        fwrite(t_end, sizeof(double), 1, fp);
        fwrite(t_file, sizeof(double), 1, fp);
        fwrite(tau, sizeof(double), 1, fp);

        fwrite(itermax, sizeof(int), 1, fp);
        fwrite(epsilon, sizeof(double), 1, fp);
        fwrite(omega, sizeof(double), 1, fp);
        fwrite(gamma, sizeof(double), 1, fp);

        fwrite(gx, sizeof(double), 1, fp);
        fwrite(gy, sizeof(double), 1, fp);
        fwrite(re, sizeof(double), 1, fp);

        fwrite(w_left, sizeof(int), 1, fp);
        fwrite(w_right, sizeof(int), 1, fp);
        fwrite(w_top, sizeof(int), 1, fp);
        fwrite(w_bottom, sizeof(int), 1, fp);

        fwrite(dx, sizeof(double), 1, fp);
        fwrite(dy, sizeof(double), 1, fp);
        fwrite(nx, sizeof(int), 1, fp);
        fwrite(ny, sizeof(int), 1, fp);

        for (i=0; i<*nx+2; i++) {
            for (j=0; j<*ny+2; j++) {
                tmp = (*P)[i][j];
                fwrite(&tmp, sizeof(double), 1, fp);
            }
        }

        for (i=0; i<*nx+2; i++) {
            for (j=0; j<*ny+2; j++) {
                tmp = (*U)[i][j];
                fwrite(&tmp, sizeof(double), 1, fp);
            }
        }

        for (i=0; i<*nx+2; i++) {
            for (j=0; j<*ny+2; j++) {
                tmp = (*V)[i][j];
                fwrite(&tmp, sizeof(double), 1, fp);
            }
        }

        fclose(fp);
    }
    return 0;
}
